@@ -1,3 +1,4 @@
+import asyncio
 import calendar
 import collections
 import contextlib
@@ -105,12 +106,15 @@ class DataConnectionThrottleStreamIO(ThrottleStreamIO):
 
 class BaseClient:
 
-    def __init__(self, *, socket_timeout=None,
+    def __init__(self, *,
+                 socket_timeout=None,
+                 connection_timeout=None,
                  read_speed_limit=None, write_speed_limit=None,
                  path_timeout=None, path_io_factory=pathio.PathIO,
                  encoding="utf-8", ssl=None, parse_list_line_custom=None,
                  passive_commands=("epsv", "pasv"), **siosocks_asyncio_kwargs):
         self.socket_timeout = socket_timeout
+        self.connection_timeout = connection_timeout
         self.throttle = StreamThrottle.from_limits(
             read_speed_limit,
             write_speed_limit,
@@ -128,7 +132,10 @@ class BaseClient:
     async def connect(self, host, port=DEFAULT_PORT):
         self.server_host = host
         self.server_port = port
-        reader, writer = await self._open_connection(host, port)
+        reader, writer = await asyncio.wait_for(
+            self._open_connection(host, port),
+            self.connection_timeout,
+        )
         self.stream = ThrottleStreamIO(
             reader,
             writer,
@@ -162,7 +169,7 @@ class BaseClient:
             self.stream.close()
             raise ConnectionResetError
         s = line.decode(encoding=self.encoding).rstrip()
-        logger.info(s)
+        logger.debug(s)
         return Code(s[:3]), s[3:]
 
     async def parse_response(self):
@@ -245,9 +252,9 @@ class BaseClient:
                 # Censor the user's command
                 raw = command[:censor_after]
                 stars = "*" * len(command[censor_after:])
-                logger.info("%s%s", raw, stars)
+                logger.debug("%s%s", raw, stars)
             else:
-                logger.info(command)
+                logger.debug(command)
             message = command + END_OF_LINE
             await self.stream.write(message.encode(encoding=self.encoding))
         if expected_codes or wait_codes:
@@ -557,6 +564,9 @@ class Client(BaseClient):
     :param socket_timeout: timeout for read operations
     :type socket_timeout: :py:class:`float`, :py:class:`int` or `None`
 
+    :param connection_timeout: timeout for connection
+    :type connection_timeout: :py:class:`float`, :py:class:`int` or `None`
+
     :param read_speed_limit: download speed limit in bytes per second
     :type server_to_client_speed_limit: :py:class:`int` or `None`
 
